import os, sys
try:
    from unittest import mock
except ImportError:
    import mock

import pptshape.directive
try:
    from pptshape import ppt
except ImportError:
    ppt = None

TESTDIR = os.path.split(__file__)[0]

def test_open():
    if not ppt:
        return

    p = ppt.PPTShape(os.path.join(TESTDIR, 'testppt.pptx'))
    shape = p.saveShape('assign_list', 
                os.path.join(TESTDIR, 'assign.png'))

    p.quit()

def test_update():
    if not ppt:
        return

    NAME_PNG = 'abc.png'
    path_png = os.path.join(TESTDIR, NAME_PNG)

    NAME_PPT = 'testppt.pptx'
    path_ppt = os.path.join(TESTDIR, NAME_PPT)

    directive = pptshape.directive.PPTShape(None, None, None, 
             None, None, None, None, None, None)
    if os.path.exists(path_png):
        os.unlink(path_png)

    directive.arguments = [NAME_PNG]
    directive.state = mock.Mock()
    directive.state.document.current_source = os.path.join(TESTDIR, 'test.rst')
    directive.options = {
        'pptfilename': NAME_PPT,
        'shapename': 'shape-title',
    }
    directive.run()
    assert os.path.exists(path_png)
    
    os.utime(path_png, (0, 0))
    directive.run()

    assert os.stat(path_png).st_mtime != 0

    # Make png file empty.
    open(path_png, 'w').close()

    directive.run()
    assert os.path.getsize(path_png) == 0




import os
from docutils.parsers.rst import directives
from docutils.parsers.rst.directives import images
try:
    from . import ppt
except ImportError:
    ppt = None

class PPTShape(images.Image):
    option_spec = images.Image.option_spec.copy()
    option_spec['pptfilename'] = directives.unchanged_required
    option_spec['shapename'] = directives.unchanged_required

    def run(self):
        if ppt:
            dirname = os.path.split(self.state.document.current_source)[0]
            filename = os.path.join(dirname, self.options['pptfilename'])
            shapename = self.options['shapename']
            imagename = os.path.join(dirname, self.arguments[0])

            # check if ppt was updated.
            mtime_img = 0
            if os.path.exists(imagename):
                mtime_img = os.stat(imagename).st_mtime
            # TODO: How to emit error message?
            if os.stat(filename).st_mtime > mtime_img:  
                pptfile = ppt.open(filename)
                if pptfile:
                    pptfile.saveShape(shapename, imagename)
                pptfile.quit()

        return super(PPTShape, self).run()


def setup(app):
    app.add_directive('ppt-shape', PPTShape)



import pywintypes
import win32com.client, win32com.client.gencache, win32com.gen_py

def open(filename):
    try:
        return PPTShape(filename)
    except pywintypes.com_error as exc:
        if exc.hresult == -2147221005: # Invalid prog-id
            return None
        raise

class PPTShape:
    def __init__(self, filename):
        self.ppt = win32com.client.gencache.EnsureDispatch("PowerPoint.Application") 
        #self.ppt.Visible = 0
        self.filename = filename
        self.presentation = self.ppt.Presentations.Open(self.filename)

    def quit(self):
        self.ppt.Quit()
        self.ppt = None

    def shapes(self):
        for slide in self.presentation.Slides:
            for shape in slide.Shapes:
                yield shape

    def findShape(self, name):
        for shape in self.shapes():
            if shape.Title == name:
                return shape
    
    def saveShape(self, name, filename):
        shape = self.findShape(name)
        if not shape:
            raise ValueError(
                    "Shape '{}' doesn't found in {}".format(name, self.filename))
        #ppRelativeToSlide
        #ppClipRelativeToSlide
        #ppScaleToFit
        #ppScaleXY
        if shape:
            SCALE = 4   # Expand size by 4 to texts to be anti-aliased.

            # ScaleWidth and ScaleHeight are dimentions of slide in ppScaleXY
            # mode.
            w = self.presentation.SlideMaster.Width 
            h = self.presentation.SlideMaster.Height

            shape.Export(filename, 
                Filter=win32com.client.constants.ppShapeFormatPNG,
                ScaleWidth=w*SCALE, ScaleHeight=h*SCALE,
                ExportMode= win32com.client.constants.ppScaleXY)





import os
from setuptools import setup

def read(fname):
    return open(
            os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "pptshape",
    version = "0.0.3",
    author = "Atsuo Ishimoto",
    author_email = "ishimoto@gembook.org",
    description = "Extract images from PowerPoint presentation files for Sphinx.",
    license = "BSD",
    url = "https://github.com/atsuoishimoto/pptshape",
    long_description=read('README.rst'),
    install_requires=[
        'sphinx',
        ],
    packages=['pptshape'],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Topic :: Software Development :: Documentation",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
    ],
)
